/*
 * CAColor.java
 *
 * Created on May 15, 2003
 */

/**
 * @version 1.0 
 * @author  Harry Fu
 */

import java.awt.*;

/**
 * CAColor create color arrays for CA color options
 */
public class CAColor {
   
   private static final short CLR_SPACE = 360;
   
   /** Creates a  instance of CAColor */
   public CAColor() {
   }
   /**
    * getHSBColor creates the HSB color spectrum for discrete CA
    * @param stateCt number of states 
    */
   public Color[] getHSBColor(int stateCt) {
      Color[] colorHSB = new Color[stateCt];
      colorHSB[0] = Color.white;
      float hue;
      if (stateCt > 1) {
         for (int i = 1; i < stateCt; i++ ) {
            hue = (i - 1.5F)/(stateCt - 1.5F);
            colorHSB[i] = Color.getHSBColor(hue, 1.0F, 1.0F);
         }
      }
      return colorHSB;
   } // end getHSBColor
   /**
    * getGrayscaleColor creates a grayscale color spectrum for discrete CA
    * @param stateCt number of states
    */
   public Color[] getGrayscaleColor(int stateCt) {
      Color[] colorGrayscale =  new Color[stateCt];
      // Generate the colors and store them in the color arrays.     
      colorGrayscale[0] = Color.white;
      colorGrayscale[1] = Color.black;
      if (stateCt > 2) {
         for (int i = 2; i < stateCt; i++) {
            /* As i goes from 1 to cvColors, this color goes from almost white to black.
               from white to black */
            colorGrayscale[i] = new Color((float)((double)(stateCt-i)/(double)stateCt), 
                                          (float)((double)(stateCt-i)/(double)stateCt), 
                                          (float)((double)(stateCt-i)/(double)stateCt));
         }
      }
      return colorGrayscale;
   } // end getGrayscaleColor()
   /**
    * getRainbowColor() creates a HSB color spectrum (Rainbow Color) for discrete CA
    * @param stateCt number of states
    */
   public Color[] getRainbowColor(int stateCt) {
      Color[] colorRainbow =  new Color[stateCt];
      // Generate the colors and store them in the color arrays.     
      colorRainbow[0] = Color.white;
      float hue;
      float saturation;
      float brightness;
      if (stateCt > 1) {
         for (int i = 1; i < stateCt; i++) {
         hue = (float)((double)(i)/(double)(stateCt));
         saturation = (float)((double)(stateCt-i)/(double)(stateCt)); 
         brightness = (float)((double)(stateCt-i)/(double)(stateCt)); // rainbow color
         colorRainbow[i] = Color.getHSBColor(hue, saturation, brightness); 
         }
      }
      return colorRainbow;
   } // end getRainbowColor()
   /**
    * getMatrixColor() creates a HSB color spectrum (The Matrix Color) for discrete CA
    * @param stateCt number of states
    */
   public Color[] getMatrixColor(int stateCt) {
      Color[] colorMatrix =  new Color[stateCt];
      // Generate the colors and store them in the color arrays.     
      colorMatrix[0] = Color.white;
      float hue;
      float saturation;
      float brightness;
      if (stateCt > 1) {
         for (int i = 1; i < stateCt; i++) {
         hue = (float)((double)(stateCt-i)/(double)(stateCt));
         saturation = (float)((double)(stateCt-i)/(double)(stateCt)); 
         brightness = (float)((double)(i)/(double)(stateCt)); // the matrix color;
         colorMatrix[i] = Color.getHSBColor(hue, saturation, brightness); 
         }
      }
      return colorMatrix;
   } // end getMatrixColor()
   /**
    * getCandyColor() creates a HSB color spectrum (Candy Color) for discrete CA
    * @param stateCt number of states
    */
   public Color[] getCandyColor(int stateCt) {
      Color[] colorCandy =  new Color[stateCt];
      // Generate the colors and store them in the color arrays.     
      colorCandy[0] = Color.white;
      float hue;
      float saturation;
      if (stateCt > 1) {
         for (int i = 1; i < stateCt; i++) {
         hue = (float)((double)(i)/(double)(stateCt));
         saturation = (float)((double)(stateCt-i)/(double)(stateCt)); // candy color
         colorCandy[i] = Color.getHSBColor(hue, saturation, 0.95F); // candy color 
         }
      }
      return colorCandy;
   } // end getCandyColor()      
   /**
    * getHSBColor creates a HSB Hue color spectrum
    */
   public Color[] getHSBColor() {
      Color[] colorHSB = new Color[CLR_SPACE];
      float hue;
      for (int i = 0; i < CLR_SPACE; i++) {
         hue = (float)((double)(i)/(double)(CLR_SPACE));
         colorHSB[i] = Color.getHSBColor(hue, 1.0F, 1.0F);
      }
      return colorHSB;
   } // end getHSBColor()
   /**
    * getGrayscaleColor creates a gradient grayscale color spectrum
    */
   public Color[] getGrayscaleColor() {
      Color[] colorGrayscale =  new Color[CLR_SPACE];
      // Generate the colors and store them in the color arrays.     
      for (int i = 0; i < CLR_SPACE; i++) {
         /* As i goes from 1 to cvColors, this color goes from almost white to black.
            from white to black */
         colorGrayscale[i] = new Color((float)((double)(CLR_SPACE-i)/(double)CLR_SPACE), 
                                       (float)((double)(CLR_SPACE-i)/(double)CLR_SPACE), 
                                       (float)((double)(CLR_SPACE-i)/(double)CLR_SPACE));
      }
      return colorGrayscale;
   } // end getGrayscaleColor()
   /**
    * getRainbowColor creates a HSB color spectrum (Rainbow Color)
    */
   public Color[] getRainbowColor() {
      Color[] colorRainbow = new Color[CLR_SPACE];
      float hue;
      float saturation;
      float brightness;
      for (int i = 0; i < CLR_SPACE; i++) {
         hue = (float)((double)(i)/(double)(CLR_SPACE));
         saturation = (float)((double)((CLR_SPACE-i))/(double)(CLR_SPACE)); 
         brightness = (float)((double)(CLR_SPACE-i)/(double)(CLR_SPACE)); // rainbow color
         colorRainbow[i] = Color.getHSBColor(hue, saturation, brightness); 
      }
      return colorRainbow;
   } // end getRainbowColor()   
   /**
    * getMatrixColor creates a HSB color spectrum (The Matrix Color) 
    */
   public Color[] getMatrixColor() {
      Color[] colorMatrix = new Color[CLR_SPACE];
      float hue;
      float saturation;
      float brightness;
      for (int i = 0; i < CLR_SPACE; i++) {
         hue = (float)((double)(CLR_SPACE-i)/(double)(CLR_SPACE));
         saturation = (float)((double)(CLR_SPACE-i)/(double)(CLR_SPACE)); 
         brightness = (float)((double)(i)/(double)(CLR_SPACE)); // forest color
         colorMatrix[i] = Color.getHSBColor(hue, saturation, brightness); 
      }
      return colorMatrix;
   } // end getMatrixColor()
  /**
    * getCandyColor creates a HSB color spectrum (Candy Color)
    */
   public Color[] getCandyColor() {
      Color[] colorCandy = new Color[CLR_SPACE];
      float hue;
      float saturation;
      for (int i = 0; i < CLR_SPACE; i++) {
         hue = (float)((double)(i)/(double)(CLR_SPACE));
         saturation = (float)((double)(CLR_SPACE-i)/(double)(CLR_SPACE)); // candy color
         colorCandy[i] = Color.getHSBColor(hue, saturation, 0.95F); // candy color
      }
      return colorCandy;
   } // end getCandyColor()  
} // end CAColor
