/*
 * CAMorph.java
 *
 * Created on May 15, 2003
 */

/**
 * @version 1.0 
 * @author  Harry Fu
 */

import java.awt.*;
import java.awt.event.*;   

import javax.swing.*;
import javax.swing.event.*;
import java.util.StringTokenizer;
import java.util.Hashtable;
/**
 * CAMorph displays 9 Cellular Automata Views  
 */
public class CAMorph extends JApplet implements Runnable {
   
   // declare CADispaly Object
   CADisplay CA1;
   CADisplay CA2; 
   CADisplay CA3; 
   CADisplay CA4; 
   CADisplay CA5; 
   CADisplay CA6; 
   CADisplay CA7; 
   CADisplay CA8;
   CADisplay CA9; 
   // declare CARule Object
   CARule caRule;                     
   private int[] rules = null;            // for copy-mutate and mutate  
   // declare CAColor Object
   private CAColor caColor = null;
   private Color[] stColor = null;        // color for CA
   private Color[] cvColor = null;        // color for CV
   private Thread CARunner = null;        // initialize CA thread
   private boolean threadSuspended;
   private JPanel mainPanel;              // create main panel for CAMorph
   private CAStatusPanel statusPanel;     // create status panel for CAMorph
   private CADisplayPanel displayPanel;   // create display panel for 9 CA views
   private CAControlPanel controlPanel;   // create control panel for button action
   // set control flags
   private JPanel CA1Panel;               // for CADisplayPanel border
   private JPanel CA2Panel;               // for CADisplayPanel border
   private JPanel CA3Panel;               // for CADisplayPanel border
   private JPanel CA4Panel;               // for CADisplayPanel border
   private JPanel CA5Panel;               // for CADisplayPanel border
   private JPanel CA6Panel;               // for CADisplayPanel border
   private JPanel CA7Panel;               // for CADisplayPanel border
   private JPanel CA8Panel;               // for CADisplayPanel border
   private JPanel CA9Panel;               // for CADisplayPanel border
   private JTextField counter;            // counter for CA cycles
   private JTextField runStatus;          // run or paused
   private JTextField focusStatus;        // focused view or all views
   private JTextField caType;             // discrete CA or continuous CA
   // set control flags
   boolean CA1Action = true; 
   boolean CA2Action = true;
   boolean CA3Action = true;
   boolean CA4Action = true;
   boolean CA5Action = true; 
   boolean CA6Action = true;
   boolean CA7Action = true;
   boolean CA8Action = true;
   boolean CA9Action = true;
   // user configurable variables
   private int interval = 30;       // set default thread sleep time to 30 ms
   private int states = 17;         // default CA (Continuous CA)
   private int neighbors = 3;       // default neighbors
   private int ruleCode = 225;      // default rule code
   private double inc = 0;          // increment value for continuous CA
   private double lw = 1;           // left weight for L cell    
   private double rw = 1;           // right weight for R cell
   private int cycles = 0;          // keep track of CA cycles
   private String seedType = "One Seed"; // seed Type for CA
   private static final String ONE_CELL = "One Seed";
   private static final String RAND_CELL = "Rand Seed";
   private static final String LINEAR_CELL = "Linear Seed";
   private static final String SINE_CELL = "Sine Seed";

   /** Initialization method that will be called after the applet is loaded
    *  into the browser.
    */
   public void init() {
      CA1 = new CADisplay();
      CA2 = new CADisplay();
      CA3 = new CADisplay();
      CA4 = new CADisplay();
      CA5 = new CADisplay();
      CA6 = new CADisplay();
      CA7 = new CADisplay();
      CA8 = new CADisplay();
      CA9 = new CADisplay();
      // creates a main panel
      mainPanel = new JPanel();
      mainPanel.setLayout(new BorderLayout(1,1));
      // creates a status panel
      statusPanel = new CAStatusPanel();
      // creates a display panel with 9 CA views
      displayPanel = new CADisplayPanel();
      // creates a control panel
      controlPanel = new CAControlPanel();      
      // set background
      setBackground(Color.gray);       
      // add panels to main panel
      getContentPane().add(statusPanel, BorderLayout.NORTH);
      getContentPane().add(displayPanel, BorderLayout.CENTER);
      getContentPane().add(controlPanel, BorderLayout.SOUTH); 
   } // end init()
   /**
    * run() Thread interface
    */
   public void run() {
      Thread thisThread = Thread.currentThread();
      updateAllView();
      while(CARunner == thisThread) {   
         if(states <= 16) {
            CA1.nextCAGeneration();
            CA2.nextCAGeneration();
            CA3.nextCAGeneration();
            CA4.nextCAGeneration();
            CA5.nextCAGeneration();
            CA6.nextCAGeneration();
            CA7.nextCAGeneration();
            CA8.nextCAGeneration();
            CA9.nextCAGeneration();
         }
         else {
            CA1.nextCVGeneration();
            CA2.nextCVGeneration();
            CA3.nextCVGeneration();
            CA4.nextCVGeneration();
            CA5.nextCVGeneration();
            CA6.nextCVGeneration();
            CA7.nextCVGeneration();
            CA8.nextCVGeneration();
            CA9.nextCVGeneration();
         }
         cycles++;   // keep track CA cycles
         counter.setText(" " + cycles);
         try {
            Thread.sleep(interval);
               synchronized(controlPanel) {
                  while (threadSuspended && CARunner == thisThread)
                     controlPanel.wait();
               }
         } catch (InterruptedException e) {
            CARunner.start();
         }
      }
   } // end run()
   /**
    * start() Applet interface
    */
   public void start() {       
      if (CARunner == null) {
         CARunner = new Thread(this);
         CARunner.start();
      }
   } // end start()
   /**
    * stop() Applet interface
    */
   public synchronized void stop() {
         CARunner = null;
            notify();   // for thread safe suspend
         CA1.disposeOffScrImg();
         CA2.disposeOffScrImg();
         CA3.disposeOffScrImg();
         CA4.disposeOffScrImg();
         CA5.disposeOffScrImg();
         CA6.disposeOffScrImg();
         CA7.disposeOffScrImg();
         CA8.disposeOffScrImg();
         CA9.disposeOffScrImg();
   } // end stop()
   /**
    * Applet Info
    */
   public String getAppletInfo() {
      return "CAMorph: 1D Cellular Automata Player\nAuthor: Harry Fu";
   }
   /**
    * updateAllView seed new CAs
    */
   public synchronized void updateAllView() {      
      cycles = 0;
      caRule = new CARule();
      
      if (neighbors > 5 && (states > 7 && states <= 16))
         states = 7;
      
      if (states <= 16) {
         caType.setText(" Discrete Valued CA");   // discrete CA type
         caRule.setRandRule(states, neighbors);
         CA1.CAConfig(states, neighbors, caRule.getRule(), stColor);
         CA1.setCAWorld(seedType);
         
         caRule.setRandRule(states, neighbors);
         CA2.CAConfig(states, neighbors, caRule.getRule(), stColor);
         CA2.setCAWorld(seedType);

         caRule.setRandRule(states, neighbors);
         CA3.CAConfig(states, neighbors, caRule.getRule(), stColor);
         CA3.setCAWorld(seedType);

         caRule.setRandRule(states, neighbors);
         CA4.CAConfig(states, neighbors, caRule.getRule(), stColor);
         CA4.setCAWorld(seedType);

         caRule.setRandRule(states, neighbors);
         CA5.CAConfig(states, neighbors, caRule.getRule(), stColor);
         CA5.setCAWorld(seedType);

         caRule.setRandRule(states, neighbors);
         CA6.CAConfig(states, neighbors, caRule.getRule(), stColor);
         CA6.setCAWorld(seedType);

         caRule.setRandRule(states, neighbors);
         CA7.CAConfig(states, neighbors, caRule.getRule(), stColor);
         CA7.setCAWorld(seedType);

         caRule.setRandRule(states, neighbors);
         CA8.CAConfig(states, neighbors, caRule.getRule(), stColor);
         CA8.setCAWorld(seedType);

         caRule.setRandRule(states, neighbors);
         CA9.CAConfig(states, neighbors, caRule.getRule(), stColor);
         CA9.setCAWorld(seedType);
      }
      else {
         caType.setText(" Continuous Valued CA");   // continuous CA type
         CA1.CVConfig(caRule.getRandInc(), lw, rw, cvColor);
         CA1.setCAWorld(seedType);

         CA2.CVConfig(caRule.getRandInc(), lw, rw, cvColor);
         CA2.setCAWorld(seedType);

         CA3.CVConfig(caRule.getRandInc(), lw, rw, cvColor);
         CA3.setCAWorld(seedType);

         CA4.CVConfig(caRule.getRandInc(), lw, rw, cvColor);
         CA4.setCAWorld(seedType);

         CA5.CVConfig(caRule.getRandInc(), lw, rw, cvColor);
         CA5.setCAWorld(seedType);

         CA6.CVConfig(caRule.getRandInc(), lw, rw, cvColor);
         CA6.setCAWorld(seedType);

         CA7.CVConfig(caRule.getRandInc(), lw, rw, cvColor);
         CA7.setCAWorld(seedType);

         CA8.CVConfig(caRule.getRandInc(), lw, rw, cvColor);
         CA8.setCAWorld(seedType);

         CA9.CVConfig(caRule.getRandInc(), lw, rw, cvColor);
         CA9.setCAWorld(seedType);
      }      
   } // end updateAllView()
   /**
    * changeColor change the color type for continuous CA
    * @param color color type array
    */
   public synchronized void changeColor(int color) {
      caColor = new CAColor();
      switch (color) {
         case 0: if (states <= 16) 
                    stColor = caColor.getRainbowColor(states);
                 else
                    cvColor = caColor.getRainbowColor();             
            break;
         case 1: if (states <= 16)
                    stColor = caColor.getHSBColor(states); 
                 else
                    cvColor = caColor.getHSBColor();                 
            break;
         case 2: if (states <= 16)
                    stColor = caColor.getGrayscaleColor(states);
                 else
                    cvColor = caColor.getGrayscaleColor();
            break;
         case 3: if (states <= 16)
                    stColor = caColor.getMatrixColor(states); 
                 else        
                    cvColor = caColor.getMatrixColor();
            break;         
         case 4: if (states <= 16)
                    stColor = caColor.getCandyColor(states); 
                 else
                    cvColor = caColor.getCandyColor();
            break;
      }
   } // end changeColor
   /**
    * getInsets the system calls this method to set pixels between components
    */ 
   public Insets getInsets() {
      return new Insets(1,1,1,1); // best Insets value for applet size WxH = 603 x 570 or 663 x 621
   }
   /**
    * initSeed set initial seed type for CA
    * @param seedType sets the seed type
    */
   public synchronized void initSeed(String seedType) {
      cycles = 0;
      if (CA1Action) {    
         CA1.setCAWorld(seedType);
      }
      if (CA2Action) {
         CA2.setCAWorld(seedType);
      }
      if (CA3Action) {
         CA3.setCAWorld(seedType);
      }
      if (CA4Action) {
         CA4.setCAWorld(seedType);
      }
      if (CA5Action) {
         CA5.setCAWorld(seedType);
      }
      if (CA6Action) {
         CA6.setCAWorld(seedType);
      }
      if (CA7Action) {
         CA7.setCAWorld(seedType);
      }
      if (CA8Action) {
         CA8.setCAWorld(seedType);
      }
      if (CA9Action) {
         CA9.setCAWorld(seedType);
      }
   }
   /**
    * applyWaveEquation applies wave equation
    * @param setWave true to apply wave equation 
    */
   public synchronized void applyWaveEquation(boolean setWave) {
      if (CA1Action) {
         CA1.setWaveEquation(setWave);
      }
      if (CA2Action) {
         CA2.setWaveEquation(setWave);
      }
      if (CA3Action) {
         CA3.setWaveEquation(setWave);
      }
      if (CA4Action) { 
         CA4.setWaveEquation(setWave);
      }
      if (CA5Action) {
         CA5.setWaveEquation(setWave);
      }
      if (CA6Action) { 
         CA6.setWaveEquation(setWave);
      }
      if (CA7Action) {
         CA7.setWaveEquation(setWave);
      }
      if (CA8Action) {
         CA8.setWaveEquation(setWave);
      }
      if (CA9Action) {
         CA9.setWaveEquation(setWave); 
      }
   }
   /**
    * CAStatusPanel Class
    */
   class CAStatusPanel extends JPanel {
      public CAStatusPanel() {
         setLayout(new GridLayout(1,4,1,1));      
         // creates counter text field 
         counter = new JTextField(" ", 0);
         counter.setEditable(false);      
         // creates run status text field 
         runStatus = new JTextField(" Status: Running");
         runStatus.setEditable(false);
         // creates focus status text field 
         focusStatus = new JTextField(" All Views");
         focusStatus.setEditable(false);      
         // creates CA-type status text field 
         caType = new JTextField(" Discrete Valued CA");
         caType.setEditable(false);
         add(runStatus);
         add(focusStatus);
         add(caType);
         add(counter);
      }
      /**
       * getInsets the system calls this method to set pixels between components
       */ 
      public Insets getInsets() {
         return new Insets(1,1,1,1); // best Insets value for applet size WxH = 603 x 570 or 663 x 621
      }
   }
   /**
    * CADisplayPanel Class
    */
   class CADisplayPanel extends JPanel implements MouseListener {
      // For Border hightlight
      private final Color BORDER_CLR = Color.gray; // default
      private final Color BORDER_LITE = Color.red; // when clicked
      
      public CADisplayPanel() {
         setLayout(new GridLayout(3,3,1,1));
         // creates a panel for CA1 Display
         CA1.addMouseListener(this);
         CA1Panel = new JPanel();
         CA1Panel.setLayout(new BorderLayout());
         CA1Panel.setBorder(BorderFactory.createLineBorder(BORDER_CLR, 1));
         CA1Panel.add(CA1);
         // creates a panel for CA2 Display
         CA2.addMouseListener(this);
         CA2Panel = new JPanel();
         CA2Panel.setLayout(new BorderLayout());
         CA2Panel.setBorder(BorderFactory.createLineBorder(BORDER_CLR, 1));
         CA2Panel.add(CA2);
         // creates a panel for CA3 Display
         CA3.addMouseListener(this);
         CA3Panel = new JPanel();
         CA3Panel.setLayout(new BorderLayout());
         CA3Panel.setBorder(BorderFactory.createLineBorder(BORDER_CLR, 1));
         CA3Panel.add(CA3);        
         // creates a panel for CA4 Display
         CA4.addMouseListener(this);
         CA4Panel = new JPanel();
         CA4Panel.setLayout(new BorderLayout());
         CA4Panel.setBorder(BorderFactory.createLineBorder(BORDER_CLR, 1));
         CA4Panel.add(CA4);         
         // creates a panel for CA5 Display      
         CA5.addMouseListener(this);
         CA5Panel = new JPanel();
         CA5Panel.setLayout(new BorderLayout());
         CA5Panel.setBorder(BorderFactory.createLineBorder(BORDER_CLR, 1));
         CA5Panel.add(CA5);
         // creates a panel for CA6 Display        
         CA6.addMouseListener(this);
         CA6Panel = new JPanel();
         CA6Panel.setLayout(new BorderLayout());
         CA6Panel.setBorder(BorderFactory.createLineBorder(BORDER_CLR, 1));
         CA6Panel.add(CA6); 
         // creates a panel for CA7 Display        
         CA7.addMouseListener(this);
         CA7Panel = new JPanel();
         CA7Panel.setLayout(new BorderLayout());
         CA7Panel.setBorder(BorderFactory.createLineBorder(BORDER_CLR, 1));
         CA7Panel.add(CA7);
         // creates a panel for CA8 Display        
         CA8.addMouseListener(this);
         CA8Panel = new JPanel();
         CA8Panel.setLayout(new BorderLayout());
         CA8Panel.setBorder(BorderFactory.createLineBorder(BORDER_CLR, 1));
         CA8Panel.add(CA8);
         // creates a panel for CA9 Display        
         CA9.addMouseListener(this);
         CA9Panel = new JPanel();
         CA9Panel.setLayout(new BorderLayout());
         CA9Panel.setBorder(BorderFactory.createLineBorder(BORDER_CLR, 1));
         CA9Panel.add(CA9);
         add(CA1Panel);
         add(CA2Panel);
         add(CA3Panel);
         add(CA4Panel);
         add(CA5Panel);
         add(CA6Panel);
         add(CA7Panel);
         add(CA8Panel);
         add(CA9Panel);
      }
      /**
       * getInsets the system calls this method to set pixels between components
       */ 
      public Insets getInsets() {
         return new Insets(1,1,1,1); // best Insets value for applet size WxH = 603 x 570 or 663 x 621
      }
      /**
       * mouseClicked
       */
      public void mouseClicked(MouseEvent evt) {
         Object source = evt.getSource();
         if ((evt.getModifiers() & MouseEvent.BUTTON1_MASK) != 0) { 
            if (source == CA1) { // Perform CA1 
               CAFocused(1);
               borderHighLite(1);
               if (evt.getClickCount() == 2) { // copy-mutate 
                  copyMutate(CA1.getDefaultRule(), CA1.getInc(), CA1.getLW(), CA1.getRW());
               }
            }
            else if (source == CA2) { // Perform CA2 
               CAFocused(2);
               borderHighLite(2);
               if (evt.getClickCount() == 2) { // copy-mutate
                  copyMutate(CA2.getDefaultRule(), CA2.getInc(), CA2.getLW(), CA2.getRW());
               }
            }
            else if (source == CA3) { // Perform CA3
               CAFocused(3);
               borderHighLite(3);
               if (evt.getClickCount() == 2) { // copy-mutate 
                  copyMutate(CA3.getDefaultRule(), CA3.getInc(), CA3.getLW(), CA3.getRW());
               }
            }
            else if (source == CA4) { // Perform CA4
               CAFocused(4);
               borderHighLite(4);
               if (evt.getClickCount() == 2) { // copy-mutate 
                  copyMutate(CA4.getDefaultRule(), CA4.getInc(), CA4.getLW(), CA4.getRW());
               }
            }
            else if (source == CA5) { // Perform CA5
               CAFocused(5);
               borderHighLite(5);
               if (evt.getClickCount() == 2) { // copy-mutate
                  copyMutate(CA5.getDefaultRule(), CA5.getInc(), CA5.getLW(), CA5.getRW());
               }
            }
            else if (source == CA6) { // Perform CA6 
               CAFocused(6);
               borderHighLite(6);
               if (evt.getClickCount() == 2) { // copy-mutate   
                  copyMutate(CA6.getDefaultRule(), CA6.getInc(), CA6.getLW(), CA6.getRW());
               }
            }
            else if (source == CA7) { // Perform CA7 
               CAFocused(7);
               borderHighLite(7);
               if (evt.getClickCount() == 2) { // copy-mutate 
                  copyMutate(CA7.getDefaultRule(), CA7.getInc(), CA7.getLW(), CA7.getRW());
               }
            }
            else if (source == CA8) { // Perform CA8
               CAFocused(8);
               borderHighLite(8);
               if (evt.getClickCount() == 2) { // copy-mutate
                  copyMutate(CA8.getDefaultRule(), CA8.getInc(), CA8.getLW(), CA8.getRW());
               }
            }
            else if (source == CA9) { // Perform CA9
               CAFocused(9);
               borderHighLite(9);
               if (evt.getClickCount() == 2) { // copy-mutate
                  copyMutate(CA9.getDefaultRule(), CA9.getInc(), CA9.getLW(), CA9.getRW());
               }
            }
         }
         else if ((evt.getModifiers() & MouseEvent.BUTTON3_MASK) != 0) { // mutate one CA
            caRule = new CARule();
            if (source == CA1) {
               CAFocused(1);
               borderHighLite(1);
               if (states <= 16) {
                  rules = new int[CA1.getDefaultRuleCt()];
                  System.arraycopy(CA1.getDefaultRule(), 0, rules, 0, CA1.getDefaultRuleCt());  
                  caRule.setMutateRule(states, neighbors, rules);            
                  CA1.CAConfig(states, neighbors, caRule.getRule(), stColor);
               }
               else {
                  CA1.CVConfig(caRule.getRandInc(), lw, rw, null);
               }
            }
            else if (source == CA2) {
               CAFocused(2);
               borderHighLite(2);
               if (states <= 16) {
                  rules = new int[CA2.getDefaultRuleCt()];
                  System.arraycopy(CA2.getDefaultRule(), 0, rules, 0, CA2.getDefaultRuleCt()); 
                  caRule.setMutateRule(states, neighbors, rules);
                  CA2.CAConfig(states, neighbors, caRule.getRule(), stColor);
               }
               else {
                  CA2.CVConfig(caRule.getRandInc(), lw, rw, null);
               }
            }
            else if (source == CA3) {
               CAFocused(3);
               borderHighLite(3);
               if (states <= 16) {
                  rules = new int[CA3.getDefaultRuleCt()];
                  System.arraycopy(CA3.getDefaultRule(), 0, rules, 0, CA3.getDefaultRuleCt());
                  caRule.setMutateRule(states, neighbors, rules);
                  CA3.CAConfig(states, neighbors, caRule.getRule(), stColor);
               }
               else {
                  CA3.CVConfig(caRule.getRandInc(), lw, rw, null);
               } 
            }
            else if (source == CA4) {
               CAFocused(4);
               borderHighLite(4);
               if (states <= 16) {
                  rules = new int[CA4.getDefaultRuleCt()];
                  System.arraycopy(CA4.getDefaultRule(), 0, rules, 0, CA4.getDefaultRuleCt());
                  caRule.setMutateRule(states, neighbors, rules);
                  CA4.CAConfig(states, neighbors, caRule.getRule(), stColor);
               }
               else {
                  CA4.CVConfig(caRule.getRandInc(), lw, rw, null);
               } 
            }
            else if (source == CA5) {
               CAFocused(5);
               borderHighLite(5);
               if (states <= 16) {
                  rules = new int[CA5.getDefaultRuleCt()];
                  System.arraycopy(CA5.getDefaultRule(), 0, rules, 0, CA5.getDefaultRuleCt());
                  caRule.setMutateRule(states, neighbors, rules);
                  CA5.CAConfig(states, neighbors, caRule.getRule(), stColor);
               }
               else {
                  CA5.CVConfig(caRule.getRandInc(), lw, rw, null);
               }
            }
            else if (source == CA6) {
               CAFocused(6);
               borderHighLite(6);
               if (states <= 16) {
                  rules = new int[CA6.getDefaultRuleCt()];
                  System.arraycopy(CA6.getDefaultRule(), 0, rules, 0, CA6.getDefaultRuleCt()); 
                  caRule.setMutateRule(states, neighbors, rules);
                  CA6.CAConfig(states, neighbors, caRule.getRule(), stColor);
               }
               else {
                  CA6.CVConfig(caRule.getRandInc(), lw, rw, null);
               }
            }
            else if (source == CA7) {
               CAFocused(7);
               borderHighLite(7);
               if (states <= 16) {
                  rules = new int[CA7.getDefaultRuleCt()];
                  System.arraycopy(CA7.getDefaultRule(), 0, rules, 0, CA7.getDefaultRuleCt());
                  caRule.setMutateRule(states, neighbors, rules);
                  CA7.CAConfig(states, neighbors, caRule.getRule(), stColor);
               }
               else {
                  CA7.CVConfig(caRule.getRandInc(), lw, rw, null);
               }
            }
            else if (source == CA8) {
               CAFocused(8);
               borderHighLite(8);
               if (states <= 16) {
                  rules = new int[CA8.getDefaultRuleCt()];
                  System.arraycopy(CA8.getDefaultRule(), 0, rules, 0, CA8.getDefaultRuleCt()); 
                  caRule.setMutateRule(states, neighbors, rules);
                  CA8.CAConfig(states, neighbors, caRule.getRule(), stColor);
               }
               else {
                  CA8.CVConfig(caRule.getRandInc(), lw, rw, null);
               }
            }
            else if (source == CA9) {
               CAFocused(9);
               borderHighLite(9);
               if (states <= 16) {
                  rules = new int[CA9.getDefaultRuleCt()];
                  System.arraycopy(CA9.getDefaultRule(), 0, rules, 0, CA9.getDefaultRuleCt()); 
                  caRule.setMutateRule(states, neighbors, rules);
                  CA9.CAConfig(states, neighbors, caRule.getRule(), stColor);
               }
               else {
                  CA9.CVConfig(caRule.getRandInc(), lw, rw, null);
               }
            }
         }
      } // end mouseClicked
      public void mouseEntered(MouseEvent evt) {
      }  
      public void mouseExited(MouseEvent evt) {
      }
      public void mousePressed(MouseEvent evt) {
      }
      public void mouseReleased(MouseEvent evt) {
      }    
      /**
       * CAFocused set action for current focusd CA
       * @param caFocus set action for current CA
       */
      private void CAFocused(int caFocus) {
         CA1Action = false;
         CA2Action = false;
         CA3Action = false;
         CA4Action = false;
         CA5Action = false;
         CA6Action = false;
         CA7Action = false;
         CA8Action = false;
         CA9Action = false;
         focusStatus.setText(" Focused View"); // for status panel
         switch (caFocus) {
            case 1: CA1Action = true;
               break;
            case 2: CA2Action = true;
               break;
            case 3: CA3Action = true;
               break;
            case 4: CA4Action = true;
               break;
            case 5: CA5Action = true;
               break;
            case 6: CA6Action = true;
               break;
            case 7: CA7Action = true;
               break;
            case 8: CA8Action = true;
               break;
            case 9: CA9Action = true;
               break;            
         }
      }
      /**
       * borderHighLite highlights the border for focused CA view
       * @param border current border being highlighted
       */
      private void borderHighLite(int border) {
         CA1Panel.setBorder(BorderFactory.createLineBorder(BORDER_CLR, 1));
         CA2Panel.setBorder(BorderFactory.createLineBorder(BORDER_CLR, 1));
         CA3Panel.setBorder(BorderFactory.createLineBorder(BORDER_CLR, 1));
         CA4Panel.setBorder(BorderFactory.createLineBorder(BORDER_CLR, 1));
         CA5Panel.setBorder(BorderFactory.createLineBorder(BORDER_CLR, 1));
         CA6Panel.setBorder(BorderFactory.createLineBorder(BORDER_CLR, 1));
         CA7Panel.setBorder(BorderFactory.createLineBorder(BORDER_CLR, 1));
         CA8Panel.setBorder(BorderFactory.createLineBorder(BORDER_CLR, 1));
         CA9Panel.setBorder(BorderFactory.createLineBorder(BORDER_CLR, 1));
         switch(border) {
            case 1: CA1Panel.setBorder(BorderFactory.createLineBorder(BORDER_LITE, 1));
               break;
            case 2: CA2Panel.setBorder(BorderFactory.createLineBorder(BORDER_LITE, 1));
               break;
            case 3: CA3Panel.setBorder(BorderFactory.createLineBorder(BORDER_LITE, 1));
               break;
            case 4: CA4Panel.setBorder(BorderFactory.createLineBorder(BORDER_LITE, 1));
               break;
            case 5: CA5Panel.setBorder(BorderFactory.createLineBorder(BORDER_LITE, 1));
               break;
            case 6: CA6Panel.setBorder(BorderFactory.createLineBorder(BORDER_LITE, 1));
               break;
            case 7: CA7Panel.setBorder(BorderFactory.createLineBorder(BORDER_LITE, 1));
               break;
            case 8: CA8Panel.setBorder(BorderFactory.createLineBorder(BORDER_LITE, 1));
               break;
            case 9: CA9Panel.setBorder(BorderFactory.createLineBorder(BORDER_LITE, 1));
               break;
         }    
      }
      /**
       * copyMutate process copy-mutate routine
       * @param rules rule for discrete CA
       * @param inc increment for continuous CA
       * @param lw left weight for continuous CA
       * @param rw right weight for continuous CA
       */
      private void copyMutate(int[] rules, double inc, double lw, double rw) {
        if (states <= 16) {  // discrete CA
            CA1.CAConfig(states, neighbors, rules, stColor);
            CA2.CAConfig(states, neighbors, rules, stColor);
            CA3.CAConfig(states, neighbors, rules, stColor);
            CA4.CAConfig(states, neighbors, rules, stColor);
            CA5.CAConfig(states, neighbors, rules, stColor);
            CA6.CAConfig(states, neighbors, rules, stColor);
            CA7.CAConfig(states, neighbors, rules, stColor);
            CA8.CAConfig(states, neighbors, rules, stColor);
            CA9.CAConfig(states, neighbors, rules, stColor);
         }
         else {   // continuous CA
            CA1.CVConfig(inc, lw, rw, null);
            CA2.CVConfig(inc, lw, rw, null);
            CA3.CVConfig(inc, lw, rw, null);
            CA4.CVConfig(inc, lw, rw, null);
            CA5.CVConfig(inc, lw, rw, null);
            CA6.CVConfig(inc, lw, rw, null);
            CA7.CVConfig(inc, lw, rw, null);
            CA8.CVConfig(inc, lw, rw, null);
            CA9.CVConfig(inc, lw, rw, null);
         }
      }
   }
   /**
    * CAControlPanel Class
    */
   class CAControlPanel extends JPanel implements ActionListener, ChangeListener {

      private JButton reseedButton;     // reseed CA button
      private JButton pauseButton;      // pause CA button for "II" or ">"
      private JButton randRuleButton;   // randomize rules for CA
      private JButton allViewButton;    // set focus for all views
      private JComboBox initCell;       // one cell/random cells/sine cells 
      private JComboBox stateK;         // select number of state K 
      private JComboBox neighborN;      // select number of neighbors N
      private JComboBox colorType;      // color type for continuous CA
      private JTextField ruleInput;     // rule input from user
      private JSlider caSpeed;          // speed motion control for CA animation

      /** Creates a new instance of CAControlPanel */
      public CAControlPanel() {
         caColor = new CAColor();
         stColor = caColor.getRainbowColor(states);
         cvColor = caColor.getRainbowColor();
         setLayout(new GridLayout(2,4,1,1));        
         // creates a restart button
         reseedButton = new JButton("Reseed");
         reseedButton.setFont(new Font("Helvetica", Font.PLAIN, 12));
         reseedButton.addActionListener(this);
         // create a pause button
         pauseButton = new JButton("\u275A\u275A"); // BLACK VERTICLE BLOCKS
         pauseButton.addActionListener(this);
         // create a text input for rule number
         ruleInput = new JTextField(""); // set an example rule for CV
         ruleInput.addActionListener(this);
         // create a random rule button
         randRuleButton = new JButton("Random Rule");
         randRuleButton.setFont(new Font("Helvetica", Font.PLAIN, 12));
         randRuleButton.addActionListener(this);
         // create combobox menu for One Cells/Random Cells
         String[] initCellItems = {"One Cell", "Random Cells", "Linear Cells",
                                   "Sine Wave Cells", "Diffusion", "Wave Equation"};
         initCell = new JComboBox(initCellItems);
         initCell.setFont(new Font("Helvetica", Font.PLAIN, 12));
         initCell.setSelectedIndex(0);
         initCell.addActionListener(this);
         // create a all views button 
         allViewButton = new JButton("All Views");
         allViewButton.setFont(new Font("Helvetica", Font.PLAIN, 12));
         allViewButton.addActionListener(this);
         // create combobox menu for state    
         String[] stateItems = { "2 States",  "3 States",  "4 States",  "5 States",
                                 "6 States",  "7 States",  "8 States",  "9 States",
                                "10 States", "11 States", "12 States", "13 States",
                                "14 States", "15 States", "16 States", "Continuous CA"};
         stateK = new JComboBox(stateItems);
         stateK.setFont(new Font("Helvetica", Font.PLAIN, 12));
         stateK.setSelectedIndex(15);
         stateK.addActionListener(this);
         // create combobox menu for neighbor (Radius 1 = 3 neighbors)
         String[] neighborItems = {"Radius 1", "Radius 2", "Radius 3"};
         neighborN = new JComboBox(neighborItems);
         neighborN.setFont(new Font("Helvetica", Font.PLAIN, 12));
         neighborN.setSelectedIndex(0);
         neighborN.addActionListener(this);
         neighborN.setEnabled(false);
         // create combobox menu for speed of CA (Default = 30)
         caSpeed = new JSlider(JSlider.HORIZONTAL, 0, 60, 30);
         caSpeed.addChangeListener(this);
         JLabel slowLabel = new JLabel("Slow");
         slowLabel.setFont(new Font("Helvetica", Font.PLAIN, 9));
         JLabel fastLabel = new JLabel("Fast");
         fastLabel.setFont(new Font("Helvetica", Font.PLAIN, 9));
         Hashtable speedLabelTable = new Hashtable();
         speedLabelTable.put(new Integer(0), slowLabel);
         speedLabelTable.put(new Integer(60), fastLabel);
         caSpeed.setLabelTable(speedLabelTable);
         caSpeed.setPaintLabels(true);
         // create combobox menu for color choice of Continuous Value CA
         String[] colorItems = {"Rainbow Color", "HSB Color", "Grayscale Color",
                                "Matrix Color", "Candy Color"};                                                                
         colorType = new JComboBox(colorItems);
         colorType.setFont(new Font("Helvetica", Font.PLAIN, 12));
         colorType.setSelectedIndex(0);
         colorType.addActionListener(this);
         // add buttons and combobox to control panel       
         add(reseedButton);
         add(pauseButton);
         add(ruleInput);
         add(randRuleButton);
         add(allViewButton);
         add(colorType);
         add(stateK);
         add(caSpeed);
         add(neighborN);
         add(initCell);
      }
      /**
       * getInsets the system calls this method to set pixels between components
       */ 
      public Insets getInsets() {
         return new Insets(1,1,1,1); // best Insets value for applet size WxH = 603 x 570 or 663 x 621
      }
      /**
       * actionPerformed
       */ 
      public synchronized void actionPerformed(ActionEvent evt) {
         Object source = evt.getSource();
         if (source == reseedButton) {
            updateAllView();
         }
         else if (source == pauseButton) {
            pauseButton.setText("\u25BA"); // BLACK RIGHT-POINTING POINTER 
            runStatus.setText(" Status: Paused");
            threadSuspended = !threadSuspended;
            if (!threadSuspended) {
               notify();
               pauseButton.setText("\u275A\u275A"); // BLACK VERTICLE BLOCKS
               runStatus.setText(" Status: Running");
            }
         }
         else if (source == randRuleButton) {
            caRule = new CARule();
            if (states <= 16) {
                if (CA1Action) {
                   caRule.setRandRule(states, neighbors);
                   CA1.CAConfig(states, neighbors, caRule.getRule(), stColor);
                }
                if (CA2Action) {
                   caRule.setRandRule(states, neighbors);
                   CA2.CAConfig(states, neighbors, caRule.getRule(), stColor);
                }
                if (CA3Action) {
                   caRule.setRandRule(states, neighbors);
                   CA3.CAConfig(states, neighbors, caRule.getRule(), stColor);
                }
                if (CA4Action) {
                   caRule.setRandRule(states, neighbors); 
                   CA4.CAConfig(states, neighbors, caRule.getRule(), stColor);
                }
                if (CA5Action) {
                   caRule.setRandRule(states, neighbors); 
                   CA5.CAConfig(states, neighbors, caRule.getRule(), stColor);
                }
                if (CA6Action) {
                   caRule.setRandRule(states, neighbors);
                   CA6.CAConfig(states, neighbors, caRule.getRule(), stColor);
                }
                if (CA7Action) {
                   caRule.setRandRule(states, neighbors);
                   CA7.CAConfig(states, neighbors, caRule.getRule(), stColor);
                }
                if (CA8Action) {
                   caRule.setRandRule(states, neighbors);
                   CA8.CAConfig(states, neighbors, caRule.getRule(), stColor);
                }
                if (CA9Action) {
                   caRule.setRandRule(states, neighbors);
                   CA9.CAConfig(states, neighbors, caRule.getRule(), stColor);
                }
            }
            else {   // continouse CA
               if (CA1Action) { 
                  CA1.CVConfig(caRule.getRandInc(), caRule.getRandLW(), caRule.getRandRW(), null);
               }
               if (CA2Action) {
                  CA2.CVConfig(caRule.getRandInc(), caRule.getRandLW(), caRule.getRandRW(), null);
               }
               if (CA3Action) {
                  CA3.CVConfig(caRule.getRandInc(), caRule.getRandLW(), caRule.getRandRW(), null);
               }
               if (CA4Action) {
                  CA4.CVConfig(caRule.getRandInc(), caRule.getRandLW(), caRule.getRandRW(), null);
               }
               if (CA5Action) {
                  CA5.CVConfig(caRule.getRandInc(), caRule.getRandLW(), caRule.getRandRW(), null);
               }
               if (CA6Action) {
                  CA6.CVConfig(caRule.getRandInc(), caRule.getRandLW(), caRule.getRandRW(), null);
               }
               if (CA7Action) {
                  CA7.CVConfig(caRule.getRandInc(), caRule.getRandLW(), caRule.getRandRW(), null);
               }
               if (CA8Action) {
                  CA8.CVConfig(caRule.getRandInc(), caRule.getRandLW(), caRule.getRandRW(), null);
               }
               if (CA9Action) {
                  CA9.CVConfig(caRule.getRandInc(), caRule.getRandLW(), caRule.getRandRW(), null);
               }
            }
         }
         else if (source == allViewButton) {
            CA1Action = true;
            CA2Action = true;
            CA3Action = true;
            CA4Action = true;
            CA5Action = true;
            CA6Action = true;
            CA7Action = true;
            CA8Action = true;
            CA9Action = true;
            focusStatus.setText(" All Views");
            CA1Panel.setBorder(BorderFactory.createLineBorder(Color.gray, 1));
            CA2Panel.setBorder(BorderFactory.createLineBorder(Color.gray, 1));
            CA3Panel.setBorder(BorderFactory.createLineBorder(Color.gray, 1));
            CA4Panel.setBorder(BorderFactory.createLineBorder(Color.gray, 1));
            CA5Panel.setBorder(BorderFactory.createLineBorder(Color.gray, 1));
            CA6Panel.setBorder(BorderFactory.createLineBorder(Color.gray, 1));
            CA7Panel.setBorder(BorderFactory.createLineBorder(Color.gray, 1));
            CA8Panel.setBorder(BorderFactory.createLineBorder(Color.gray, 1));
            CA9Panel.setBorder(BorderFactory.createLineBorder(Color.gray, 1));
         }
         else if (source == initCell) {
            if (initCell.getSelectedIndex() == 0) {
               applyWaveEquation(false);
               seedType = ONE_CELL; // one cell as initial condition
               initSeed(seedType);
            }
            else if (initCell.getSelectedIndex() == 1) {
               applyWaveEquation(false);
               seedType = RAND_CELL; // randomize cell as initial condition
               initSeed(seedType);
            }
            else if (initCell.getSelectedIndex() == 2) {
               applyWaveEquation(false);
               seedType = LINEAR_CELL; // linear cell as initial condition
               initSeed(seedType);
            }
            else if (initCell.getSelectedIndex() == 3) {
               applyWaveEquation(false);
               seedType = SINE_CELL; // sine cell as initial condition
               initSeed(seedType);
            }
            else if (initCell.getSelectedIndex() == 4) { // apply diffusion equation value
               applyWaveEquation(false);
               updateCAValue(17, rules, 0, 1, 1); // set states to 17 for continuous CA only

            }
            else if (initCell.getSelectedIndex() == 5) {   // apply wave equation
               applyWaveEquation(true);
            }
         }
         else if (source == ruleInput) { // get rule number from user
            initCell.setSelectedIndex(0); // set initial condition to one cell when user input new rule
            caRule = new CARule();     
            if (states <= 16) {  // discrete CA
               try {
                  ruleCode = Integer.parseInt(ruleInput.getText());
               } catch (Exception e) {
                  ruleCode = 169;
               }
               caRule.setRule(states, neighbors, ruleCode);
               rules = caRule.getRule();
               updateCAValue(states, rules, inc, lw, rw);
            }
            else {   // continouse CA
               try {
                  StringTokenizer st = new StringTokenizer(ruleInput.getText(), "/");
                  while (st.hasMoreTokens()) {
                     inc = Double.parseDouble(st.nextToken());
                     lw = Double.parseDouble(st.nextToken());
                     rw = Double.parseDouble(st.nextToken());
                  }
                  if (inc > 1)
                     inc = 0.005;
               } catch (Exception e) {
                  inc = 0.005;
                  lw = 1;
                  rw = 1.001;
               }
               updateCAValue(states, rules, inc, lw, rw);
            }
         }
         else if (source == stateK) {
            states = stateK.getSelectedIndex() + 2;
            if (states >= 2 && states <= 16) {
               colorType.setSelectedIndex(0);   // set rainbow color for default discrete CA
               neighborN.setEnabled(true);
            }
            else if (states > 16) {
               colorType.setSelectedIndex(0);   // set rainbow color for default continous CA
               neighborN.setSelectedIndex(0);
               neighborN.setEnabled(false);
            }
            updateAllView(); 
         }
         else if (source == neighborN) {                         
            neighbors = neighborN.getSelectedIndex() * 2 + 3;
            updateAllView();
         }
         else if (source == colorType) {
            changeColor(colorType.getSelectedIndex());
            if (states <= 16) {
               CA1.CAConfig(states, neighbors, CA1.getDefaultRule(), stColor);
               CA2.CAConfig(states, neighbors, CA2.getDefaultRule(), stColor);
               CA3.CAConfig(states, neighbors, CA3.getDefaultRule(), stColor);
               CA4.CAConfig(states, neighbors, CA4.getDefaultRule(), stColor);
               CA5.CAConfig(states, neighbors, CA5.getDefaultRule(), stColor);
               CA6.CAConfig(states, neighbors, CA6.getDefaultRule(), stColor);
               CA7.CAConfig(states, neighbors, CA7.getDefaultRule(), stColor);
               CA8.CAConfig(states, neighbors, CA8.getDefaultRule(), stColor);
               CA9.CAConfig(states, neighbors, CA9.getDefaultRule(), stColor);
            }
            else {
               CA1.CVConfig(CA1.getInc(), CA1.getLW(), CA1.getRW(), cvColor);
               CA2.CVConfig(CA2.getInc(), CA2.getLW(), CA2.getRW(), cvColor);
               CA3.CVConfig(CA3.getInc(), CA3.getLW(), CA3.getRW(), cvColor);
               CA4.CVConfig(CA4.getInc(), CA4.getLW(), CA4.getRW(), cvColor);
               CA5.CVConfig(CA5.getInc(), CA5.getLW(), CA5.getRW(), cvColor);
               CA6.CVConfig(CA6.getInc(), CA6.getLW(), CA6.getRW(), cvColor);
               CA7.CVConfig(CA7.getInc(), CA7.getLW(), CA7.getRW(), cvColor);
               CA8.CVConfig(CA8.getInc(), CA8.getLW(), CA8.getRW(), cvColor);
               CA9.CVConfig(CA9.getInc(), CA9.getLW(), CA9.getRW(), cvColor);
            }
         }
      } // end actionPerformed
      public void stateChanged(ChangeEvent evt) {
         int speed = 60;
         speed = speed - caSpeed.getValue();
         if (speed == 60) 
            speed = 500;   // one step cycle 
         interval = speed;
      }
      /**
       * updateCAValue helper routine for current CA values
       */
      private void updateCAValue(int states, int[] rules, double inc, double lw, double rw) {
         if (states <= 16) {
            if (CA1Action) { 
               CA1.CAConfig(states, neighbors, rules, stColor);
            }
            if (CA2Action) {
               CA2.CAConfig(states, neighbors, rules, stColor);
            }
            if (CA3Action) {
               CA3.CAConfig(states, neighbors, rules, stColor);
            }
            if (CA4Action) {
               CA4.CAConfig(states, neighbors, rules, stColor);
            }
            if (CA5Action) {
               CA5.CAConfig(states, neighbors, rules, stColor);
            }
            if (CA6Action) {
               CA6.CAConfig(states, neighbors, rules, stColor);
            }
            if (CA7Action) {
               CA7.CAConfig(states, neighbors, rules, stColor);
            }
            if (CA8Action) {
               CA8.CAConfig(states, neighbors, rules, stColor);
            }
            if (CA9Action) {
               CA9.CAConfig(states, neighbors, rules, stColor);
            }
         }
         else {
            if (CA1Action) {
               CA1.CVConfig(inc, lw, rw, null);
            }
            if (CA2Action) {
               CA2.CVConfig(inc, lw, rw, null);
            }
            if (CA3Action) {
               CA3.CVConfig(inc, lw, rw, null);
            }
            if (CA4Action) {
               CA4.CVConfig(inc, lw, rw, null);
            }
            if (CA5Action) {
               CA5.CVConfig(inc, lw, rw, null);
            }
            if (CA6Action) {
               CA6.CVConfig(inc, lw, rw, null);
            }
            if (CA7Action) {
               CA7.CVConfig(inc, lw, rw, null);
            }
            if (CA8Action) {
               CA8.CVConfig(inc, lw, rw, null);
            }
            if (CA9Action) {
               CA9.CVConfig(inc, lw, rw, null);
            }   
         }
      }
   } // end Class CAControlPanel
} // end Class CAMorph
